package mdp;

import java.util.ArrayList;

public class Transition {
	public int action;
	public int[] states;
	public double[] probs;

	public Transition(int action, int[] states, double[] probs) {
		this.action = action;
		this.states = states;
		this.probs = probs;
	}
	public Transition(int action, int size) {
		this.action = action;
		this.states = new int[size];
		this.probs = new double[size];
	}
	
	public double statesProb(ArrayList<Integer> target) {
		// ATTENTION: states and target should be sorted
		int i = 0, j = 0;
		double sum = 0;
		int size = target.size();
		while(i < states.length && j < size) {
			if(states[i] == target.get(j)) {
				sum += probs[i];
				i++;
				j++;
			} else if (states[i] < target.get(j)){
				i++;
			} else {
				j++;
			}
		}
		return sum;
	}
	
	public String toString() {
		String s = action + " -> [";
		for(int i = 0; i < states.length; i++)
			s += (i > 0 ? ", " : "") + probs[i] + ": " + states[i];
		s += "]";
		return s;
	}
	
	public static Transition dirac(int action, int state) {
		double probs[] = { 1 };
		int states[] = { state };
		return new Transition(action, states, probs);
	}
}
