package probichecker;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;

import mdp.Bisimulation;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import parser.ParseException;
import parser.Parser;
import parser.TokenMgrError;
import proc.Proc;


public class GUI {
	
	static final String windowTitle = "probichecker";
	
	Shell shell;
	Text text;
	
	public static void main(String[] args) {
		new GUI();
	}
		
	public GUI() {

		Display display = new Display();
		shell = new Shell(display);
		shell.setText(windowTitle);
		shell.setLocation(100, 100);
		
		shell.setLayout(new FormLayout());
		
		// Set menus
		//
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);
		MenuItem file = new MenuItem(menu, SWT.CASCADE);
		file.setText("File");
		
		Menu filemenu = new Menu(shell, SWT.DROP_DOWN);
		file.setMenu(filemenu);

		MenuItem openItem = new MenuItem(filemenu, SWT.PUSH);
		openItem.setText("Open");
		openItem.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event e) {
		    	  showOpenFileDialog();
		      }
		});
		new MenuItem(filemenu, SWT.SEPARATOR);

		MenuItem exitItem = new MenuItem(filemenu, SWT.PUSH);
		exitItem.setText("Exit");
		exitItem.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event e) {
		    	  shell.close();
		      }
		});

		// Set textbox
		//
		text = new Text(shell, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		FormData fd = new FormData();
		fd.left = new FormAttachment(0, 3);
		fd.right = new FormAttachment(100, -170);
		fd.top = new FormAttachment(0, 7);
		fd.bottom = new FormAttachment(100, -3);
		text.setLayoutData(fd);
		
		// set composite with buttons
		//
		Composite buttonsComp = new Composite(shell, SWT.NONE);
		fd = new FormData();
		fd.left = new FormAttachment(text, 5, SWT.RIGHT);
		fd.right = new FormAttachment(100, -3);
		fd.top = new FormAttachment(text, 0, SWT.TOP);
		fd.bottom = new FormAttachment(text, 0 , SWT.BOTTOM);
		buttonsComp.setLayoutData(fd);
		//buttonsComp.setBackground(new Color(display, 0, 0, 0));
				
		buttonsComp.setLayout(new FormLayout());
		
		Button button1 = new Button(buttonsComp, SWT.PUSH);
		button1.setText("Compute transitions");
		button1.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				computeTransitions();
			}
		});
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.right = new FormAttachment(100, 0);
		fd.top = new FormAttachment(0, 0);
		fd.bottom = new FormAttachment(0, 40);
		button1.setLayoutData(fd);

		Button button2 = new Button(buttonsComp, SWT.PUSH);
		button2.setText("Check bisimilarity");
		button2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				checkBisimilarity();
			}
		});
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.right = new FormAttachment(100, 0);
		fd.top = new FormAttachment(button1, 4, SWT.BOTTOM);
		fd.bottom = new FormAttachment(button2, 40, SWT.TOP);
		button2.setLayoutData(fd);
		
		// configure text area for drag and drop
		DropTarget dt = new DropTarget(text, DND.DROP_DEFAULT | DND.DROP_MOVE );
		dt.setTransfer(new Transfer[] { FileTransfer.getInstance() });
		dt.addDropListener(new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {
				String fileList[] = null;
				FileTransfer ft = FileTransfer.getInstance();
				if(ft.isSupportedType(event.currentDataType)) {
					fileList = (String[])event.data;
					if(fileList.length > 0)
						openFile(fileList[0]);
				}
			}
		});
		
		// open window and wait for it to close
		//
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) display.sleep();
		}
		display.dispose();
	}
	
	private void showOpenFileDialog() {
		String[] filterExtensions = {"*"};
		FileDialog fileDialog = new FileDialog(shell, SWT.OPEN);
		fileDialog.setText("Select file to open");
		//fileDialog.setFilterPath("C:/");
		fileDialog.setFilterExtensions(filterExtensions);
		String selectedFile = fileDialog.open();
		
		// stop if cancel was pressed
		if(selectedFile != null)		
			this.openFile(selectedFile);
	}
	
	private void openFile(String file) {
		try {
			String content = readTextFile(file);
			text.setText(content);
		} catch (IOException e) {
			showSimpleMessage("The file '" + file + "' could not be opened.");
			return;
		}
	}
	
	private void showSimpleMessage(String msg) {
		MessageBox messageBox = new MessageBox(shell, SWT.OK);
		messageBox.setText(windowTitle);
		messageBox.setMessage(msg);
		messageBox.open();
	}
	
	private Parser getParser() {
		String s = text.getText();
		if(s.equals("")) {
			showSimpleMessage("The text area is empty.\n\nYou must enter a process or load from a file.");
			return null;
		}
		
		StringReader sr = new StringReader(s);
		return new Parser(sr);
	}
	
	private void computeTransitions() {
		Parser p = getParser();
		if(p == null) return;
		
		try {
			HashMap<String, Proc> procTable  = p.Start();
			Proc proc = procTable.get("P1");
			if(proc == null) {
				showSimpleMessage("You must define a process named P1 whose transitions will be computed");
				return;
			}
			
			String s;
			s = "P1 = " + proc + "\n\nTransitions:\n";
			for(proc.Transition t : proc.transitions())
				s += "  " + t + "\n";
			
			showSimpleMessage(s);
			
		} catch(ParseException e) {
			showSimpleMessage("Could not parse the input:\n\n" + e.getMessage());
		} catch(TokenMgrError e) {
			showSimpleMessage("Could not parse the input:\n\n" + e.getMessage());
		}	
	}
	
	private void checkBisimilarity() {
		Parser p = getParser();
		if(p == null) return;
		
		try {
			HashMap<String,Proc> procs = p.Start();
			
			Proc proc1 = procs.get("P1"),
				 proc2 = procs.get("P2");
			if(proc1 == null || proc2 == null) {
				showSimpleMessage("You must define processes P1 and P2 which will be compared");
				return;
			}

			System.out.println("parsed:\n" + proc1 + "\n" + proc2);
	
			System.out.println("translating processes to mdp");
			Proc.clearTables();
			proc1.translateToStates();
			proc2.translateToStates();
			mdp.MDP mdp = Proc.createMDP();
			
			int state1 = proc1.getStateId();
			int state2 = proc2.getStateId();
			

			//System.out.println("\nstate translation:\n");
			//for(int i = 0; i < Proc.idToProc.size(); i++)
			//	System.out.println(i + ": " + Proc.idToProc.get(i));
			
			//System.out.println("\naction translation:\n");
			//for(int i = 0; i < Proc.idToAction.size(); i++)
			//	System.out.println(i + ": " + Proc.idToAction.get(i));
			
			//System.out.println("mdp:\n" + mdp);
			
			System.out.println("mdp generated, " + mdp.stateNo + " states");
			System.out.println("computing bisimulation classes...");
			Bisimulation bisim = new Bisimulation(mdp);
			bisim.computeClasses();
			
			boolean res = bisim.isBisimilar(state1, state2);
			String s =
				"generated " +  mdp.stateNo + " states\n\n" +
				"computed " + bisim.partitions.size() + " bisimulation classes\n\n" +
				"states " + state1 + ", " + state2 + " are " + (res ? "" : "NOT ") + "bisimilar";
			showSimpleMessage(s);

		} catch(ParseException e) {
			showSimpleMessage("Could not parse the input:\n\n" + e.getMessage());
		} catch(TokenMgrError e) {
			showSimpleMessage("Could not parse the input:\n\n" + e.getMessage());
		}				
	}
	
	
	public static String readTextFile(String fullPathFilename) throws IOException {
		StringBuffer sb = new StringBuffer(1024);
		BufferedReader reader = new BufferedReader(new FileReader(fullPathFilename));
				
		char[] chars = new char[1024];
		int numRead = 0;
		while( (numRead = reader.read(chars)) > -1) {
			String s = String.valueOf(chars);
			if(numRead < 1024)
				s = s.substring(0, numRead);
			sb.append(s);	
		}

		reader.close();

		return sb.toString();
	}
}
